@@ -1,29 +1,27 @@
 from pipelime.commands.general import (
-    TimeItCommand,
-    StageTimingCommand,
-    PipeCommand,
     CloneCommand,
     ConcatCommand,
-    ZipCommand,
-    AddRemoteCommand,
-    RemoveRemoteCommand,
-    ValidateCommand,
+    CopySharedItemsCommand,
+    FilterCommand,
+    FilterDuplicatesCommand,
     MapCommand,
     MapIfCommand,
-    SortCommand,
-    FilterCommand,
-    SliceCommand,
+    PipeCommand,
     SetMetadataCommand,
-    FilterDuplicatesCommand,
-    CopySharedItemsCommand,
+    SliceCommand,
+    SortCommand,
+    StageTimingCommand,
+    TimeItCommand,
+    ValidateCommand,
+    ZipCommand,
 )
+from pipelime.commands.piper import DrawCommand, RunCommand, WatchCommand
+from pipelime.commands.resume import ResumeCommand
 from pipelime.commands.shell import ShellCommand
 from pipelime.commands.split_ops import (
     SplitByQueryCommand,
     SplitByValueCommand,
     SplitCommand,
 )
-from pipelime.commands.toy_dataset import ToyDatasetCommand
-from pipelime.commands.piper import RunCommand, DrawCommand, WatchCommand
 from pipelime.commands.tempman import TempCommand
-from pipelime.commands.resume import ResumeCommand
+from pipelime.commands.toy_dataset import ToyDatasetCommand
