@@ -1,12 +1,16 @@
 from __future__ import annotations
+
+import json
 import typing as t
+import uuid
 from pathlib import Path
-from urllib.parse import ParseResult
 
-import pydantic as pyd
+import pydantic.v1 as pyd
+from pydantic.v1.generics import GenericModel
 
-from pipelime.utils.pydantic_types import SampleValidationInterface, ItemType
 from pipelime.piper import PiperPortType
+from pipelime.utils.pydantic_types import ItemType, SampleValidationInterface, YamlInput
+
 
 class PydanticFieldMixinBase:
     # override in derived clasess
@@ -32,7 +36,13 @@ class PydanticFieldMixinBase:
 
 class PydanticFieldWithDefaultMixin(PydanticFieldMixinBase):
     @classmethod
-    def pyd_field(cls, *, description: t.Optional[str] = None, piper_port: t.Optional[PiperPortType] = None, **kwargs):
+    def pyd_field(
+        cls,
+        *,
+        description: t.Optional[str] = None,
+        piper_port: t.Optional[PiperPortType] = None,
+        **kwargs,
+    ):
         return pyd.Field(
             default_factory=cls,  # type: ignore
             description=cls._description(description),  # type: ignore
@@ -44,7 +54,12 @@ class PydanticFieldWithDefaultMixin(PydanticFieldMixinBase):
 class PydanticFieldNoDefaultMixin(PydanticFieldMixinBase):
     @classmethod
     def pyd_field(
-        cls, *, is_required: bool = True, description: t.Optional[str] = None, piper_port: t.Optional[PiperPortType] = None, **kwargs
+        cls,
+        *,
+        is_required: bool = True,
+        description: t.Optional[str] = None,
+        piper_port: t.Optional[PiperPortType] = None,
+        **kwargs,
     ):
         return pyd.Field(
             ... if is_required else None,
@@ -78,7 +93,9 @@ class GrabberInterface(PydanticFieldWithDefaultMixin, pyd.BaseModel, extra="forb
     """
 
     _default_type_description: t.ClassVar[t.Optional[str]] = "Grabber options."
-    _compact_form: t.ClassVar[t.Optional[str]] = "<num_workers>[,<prefetch>]"
+    _compact_form: t.ClassVar[t.Optional[str]] = (
+        "<num_workers>[,<prefetch>[,<allow_nested_mp>]]"
+    )
 
     num_workers: int = pyd.Field(
         0,
@@ -90,6 +107,9 @@ class GrabberInterface(PydanticFieldWithDefaultMixin, pyd.BaseModel, extra="forb
     prefetch: pyd.PositiveInt = pyd.Field(
         2, description="The number of samples loaded in advanced by each worker."
     )
+    allow_nested_mp: bool = pyd.Field(
+        False, description="Whether to allow nested multiprocessing."
+    )
 
     @classmethod
     def __get_validators__(cls):
@@ -105,15 +125,21 @@ class GrabberInterface(PydanticFieldWithDefaultMixin, pyd.BaseModel, extra="forb
             if isinstance(value, int):
                 data["num_workers"] = value
             else:
-                wrks, _, pf = str(value).partition(",")
-                if wrks:
-                    data["num_workers"] = int(wrks)
-                if pf:
-                    data["prefetch"] = int(pf)
+                raw_data = str(value).split(",")
+                try:
+                    if raw_data[0]:
+                        data["num_workers"] = int(raw_data[0])
+                    if len(raw_data) > 1 and raw_data[1]:
+                        data["prefetch"] = int(raw_data[1])
+                    if len(raw_data) > 2 and raw_data[2]:
+                        data["allow_nested_mp"] = raw_data[2].lower() == "true"
+                except ValueError:
+                    raise ValueError("Invalid grabber definition.")
             value = data
 
         if isinstance(value, t.Mapping):
             return GrabberInterface(**value)
+
         raise ValueError("Invalid grabber definition.")
 
     def grab_all(
@@ -162,9 +188,11 @@ class GrabberInterface(PydanticFieldWithDefaultMixin, pyd.BaseModel, extra="forb
             sample_fn=sample_fn,
             size=size,
             grab_context_manager=grab_context_manager,
-            worker_init_fn=None
-            if grab_context_manager is None
-            else deepcopy(grab_context_manager).__enter__,
+            worker_init_fn=(
+                None
+                if grab_context_manager is None
+                else deepcopy(grab_context_manager).__enter__
+            ),
         )
 
     def grab_all_wrk_init(
@@ -207,7 +235,10 @@ class GrabberInterface(PydanticFieldWithDefaultMixin, pyd.BaseModel, extra="forb
         from pipelime.sequences import Grabber, grab_all
 
         grabber = Grabber(
-            num_workers=self.num_workers, prefetch=self.prefetch, keep_order=keep_order
+            num_workers=self.num_workers,
+            prefetch=self.prefetch,
+            keep_order=keep_order,
+            allow_nested_mp=self.allow_nested_mp,
         )
         track_fn = (
             None
@@ -264,7 +295,13 @@ class InputDatasetInterface(
     _compact_form: t.ClassVar[t.Optional[str]] = "<folder>[,<skip_empty>]"
     _default_port_type: t.ClassVar[PiperPortType] = PiperPortType.INPUT
 
-    folder: Path = pyd.Field(..., description="Dataset root folder.")
+    folder: t.Optional[Path] = pyd.Field(
+        None,
+        description=(
+            "Dataset root folder. Either `folder` is not `None` or the "
+            "`pipe` starts with a generator sequence."
+        ),
+    )
     merge_root_items: bool = pyd.Field(
         True,
         description=(
@@ -273,14 +310,41 @@ class InputDatasetInterface(
         ),
     )
     skip_empty: bool = pyd.Field(False, description="Filter out empty samples.")
+    pipe: t.Optional[YamlInput] = pyd.Field(
+        None,
+        description=(
+            "The pipeline to run or a path to a yaml/json file as "
+            "<filepath>[:<key-path>]. Either `folder` is not `None` or the "
+            "`pipe` starts with a generator sequence.\n"
+            "The pipeline is defined as a mapping or a sequence of mappings where "
+            "each key is a samples sequence operator to run, eg, `map`, `sort`, etc., "
+            "while the value gathers the arguments, ie, a single value, a sequence of "
+            "values or a keyword mapping."
+        ),
+    )
     schema_: t.Optional[SampleValidationInterface] = pyd.Field(
-        None, alias="schema", description="Sample schema validation."
+        None,
+        alias="schema",
+        description="Sample schema validation, verified after all operations.",
     )
 
     @pyd.validator("folder")
-    def resolve_folder(cls, v: Path):
-        # see https://bugs.python.org/issue38671
-        return v.resolve().absolute()
+    def resolve_folder(cls, v: t.Optional[Path]):
+        if v:
+            # see https://bugs.python.org/issue38671
+            return v.resolve().absolute()
+        return v
+
+    @pyd.validator("pipe", always=True)
+    def check_pipe_and_folder(
+        cls, v: t.Optional[YamlInput], values: t.Mapping[str, t.Any]
+    ):
+        if v is None:
+            if values.get("folder", None) is None:
+                raise ValueError("Either `folder` or `pipe` (or both) must be defined.")
+        elif not v.value or not isinstance(v.value, (t.Mapping, t.Sequence)):
+            raise ValueError(f"Invalid pipeline: {v.value}")
+        return v
 
     @classmethod
     def __get_validators__(cls):
@@ -313,13 +377,23 @@ class InputDatasetInterface(
         return not all(i.is_shared for i in x.values())
 
     def create_reader(self):
-        from pipelime.sequences import SamplesSequence
+        from pipelime.sequences import SamplesSequence, build_pipe
+
+        if self.folder is None:
+            reader = build_pipe(self.pipe.value, SamplesSequence)  # type: ignore
+        else:
+            reader = SamplesSequence.from_underfolder(
+                folder=self.folder,
+                merge_root_items=self.merge_root_items,
+                must_exist=True,
+            )
 
-        reader = SamplesSequence.from_underfolder(
-            folder=self.folder, merge_root_items=self.merge_root_items, must_exist=True
-        )
         if self.skip_empty:
-            reader = reader.filter(InputDatasetInterface.is_empty_fn)
+            reader = reader.filter(InputDatasetInterface.is_empty_fn)  # type: ignore
+
+        if self.pipe is not None and self.folder is not None:
+            reader = build_pipe(self.pipe.value, reader)  # type: ignore
+
         if self.schema_ is not None:
             reader = self.schema_.append_validator(reader)
         return reader
@@ -328,15 +402,15 @@ class InputDatasetInterface(
         return self.__piper_repr__()
 
     def __piper_repr__(self) -> str:
-        return self.folder.as_posix()
+        return (
+            self.folder.as_posix() if self.folder else f"<generator-{uuid.uuid1().hex}>"
+        )
 
 
 IDataset = InputDatasetInterface
 
 
-any_serialization_t = t.Literal[
-    "CREATE_NEW_FILE", "DEEP_COPY", "SYM_LINK", "HARD_LINK", "REMOTE_FILE"
-]
+any_serialization_t = t.Literal["CREATE_NEW_FILE", "DEEP_COPY", "SYM_LINK", "HARD_LINK"]
 any_item_t = t.Union[None, t.Literal["_"], ItemType]
 
 
@@ -439,12 +513,15 @@ class OutputDatasetInterface(
     """
 
     _default_type_description: t.ClassVar[t.Optional[str]] = "The output dataset."
-    _compact_form: t.ClassVar[
-        t.Optional[str]
-    ] = "<folder>[,<exists_ok>[,<force_new_files>]]"
+    _compact_form: t.ClassVar[t.Optional[str]] = (
+        "<folder>[,<exists_ok>[,<force_new_files>]]"
+    )
     _default_port_type: t.ClassVar[PiperPortType] = PiperPortType.OUTPUT
 
-    folder: Path = pyd.Field(..., description="Dataset root folder.")
+    folder: t.Optional[Path] = pyd.Field(
+        None,
+        description="Dataset root folder. Input `folder` and/or `pipe` must be set.",
+    )
     zfill: t.Optional[pyd.NonNegativeInt] = pyd.Field(
         None, description="Custom index zero-filling."
     )
@@ -455,24 +532,55 @@ class OutputDatasetInterface(
         default_factory=SerializationModeInterface,
         description="Serialization modes for items and keys.",
     )
+    pipe: t.Optional[YamlInput] = pyd.Field(
+        None,
+        description=(
+            "The pipeline to run or a path to a yaml/json file as "
+            "<filepath>[:<key-path>]. It cannot start with a generator sequence. "
+            "Input `folder` and/or `pipe` must be set.\n"
+            "The pipeline is defined as a mapping or a sequence of mappings where "
+            "each key is a samples sequence operator to run, eg, `map`, `sort`, etc., "
+            "while the value gathers the arguments, ie, a single value, a sequence of "
+            "values or a keyword mapping."
+        ),
+    )
     schema_: t.Optional[SampleValidationInterface] = pyd.Field(
-        None, alias="schema", description="Sample schema validation."
+        None,
+        alias="schema",
+        description="Sample schema validation, verified before any other operation.",
     )
 
     @pyd.validator("folder")
-    def resolve_folder(cls, v: Path):
-        # see https://bugs.python.org/issue38671
-        return v.resolve().absolute()
+    def resolve_folder(cls, v: t.Optional[Path]):
+        if v:
+            # see https://bugs.python.org/issue38671
+            return v.resolve().absolute()
+        return v
 
     @pyd.validator("exists_ok", always=True)
     def _check_folder_exists(cls, v: bool, values: t.Mapping[str, t.Any]) -> bool:
-        if not v and "folder" in values and values["folder"].exists():
+        if (
+            not v
+            and values.get("folder", None) is not None
+            and values["folder"].exists()
+        ):
             raise ValueError(
                 f"Trying to overwrite an existing dataset: `{values['folder']}`. "
                 "Please use `exists_ok=True` to overwrite."
             )
         return v
 
+    @pyd.validator("pipe", always=True)
+    def check_pipe_and_folder(
+        cls, v: t.Optional[YamlInput], values: t.Mapping[str, t.Any]
+    ):
+        if v is None:
+            if values.get("folder", None) is None:
+                raise ValueError("Either `folder` or `pipe` (or both) must be defined.")
+        elif not v.value or not isinstance(v.value, (t.Mapping, t.Sequence)):
+            raise ValueError(f"Invalid pipeline: {v.value}")
+        return v
+
     @classmethod
     def __get_validators__(cls):
         yield cls.validate
@@ -509,26 +617,43 @@ class OutputDatasetInterface(
         return self.serialization.get_context_manager()
 
     def append_writer(self, sequence):
-        writer = sequence.to_underfolder(
-            folder=self.folder,
-            zfill=self.zfill,
-            exists_ok=self.exists_ok,
-            key_serialization_mode=self.serialization.keys,
-        )
+        from pipelime.sequences import build_pipe
+
         if self.schema_ is not None:
-            writer = self.schema_.append_validator(writer)
-        return writer
+            sequence = self.schema_.append_validator(sequence)
+        if self.pipe is not None:
+            sequence = build_pipe(self.pipe.value, sequence)  # type: ignore
+        if self.folder is not None:
+            sequence = sequence.to_underfolder(
+                folder=self.folder,
+                zfill=self.zfill,
+                exists_ok=self.exists_ok,
+                key_serialization_mode=self.serialization.keys,
+            )
+
+        return sequence
 
     def as_pipe(self):
-        return {
-            "to_underfolder": {
-                "folder": self.folder,
-                "zfill": self.zfill,
-                "exists_ok": self.exists_ok,
-                "key_serialization_mode": self.serialization.keys,
-            },
-            **({} if self.schema_ is None else self.schema_.as_pipe()),
-        }
+        pipe_list = []
+        if self.schema_ is not None:
+            pipe_list.append(self.schema_.as_pipe())
+        if self.pipe is not None:
+            if isinstance(self.pipe.value, t.Mapping):
+                pipe_list.append(self.pipe.value)
+            else:
+                pipe_list.extend(self.pipe.value)  # type: ignore
+        if self.folder is not None:
+            pipe_list.append(
+                {
+                    "to_underfolder": {
+                        "folder": self.folder,
+                        "zfill": self.zfill,
+                        "exists_ok": self.exists_ok,
+                        "key_serialization_mode": self.serialization.keys,
+                    }
+                }
+            )
+        return pipe_list
 
     def as_input(self, **kwargs):
         """Returns an input dataset interface reading the same folder.
@@ -540,99 +665,12 @@ class OutputDatasetInterface(
         return self.__piper_repr__()
 
     def __piper_repr__(self) -> str:
-        return self.folder.as_posix()
+        return self.folder.as_posix() if self.folder else f"<pipe-{uuid.uuid1().hex}>"
 
 
 ODataset = OutputDatasetInterface
 
 
-class UrlDataModel(
-    pyd.BaseModel,
-    extra="forbid",
-    copy_on_model_validation="none",
-    underscore_attrs_are_private=True,
-):
-    """URL data model."""
-
-    scheme: str = pyd.Field(..., description="The addressing scheme, eg, `s3`.")
-    user: str = pyd.Field("", description="The user name.")
-    password: str = pyd.Field("", description="The user password.")
-    host: str = pyd.Field(..., description="The host name or ip address.")
-    port: t.Optional[pyd.NonNegativeInt] = pyd.Field(
-        None, description="The optional port number."
-    )
-    bucket: str = pyd.Field(..., description="The path to the remote data bucket.")
-    args: t.Mapping[str, str] = pyd.Field(
-        default_factory=dict, description="Optional remote-specific arguments."
-    )
-
-    def get_url(self):
-        from pipelime.remotes import make_remote_url
-
-        return make_remote_url(
-            scheme=self.scheme,
-            user=self.user,
-            password=self.password,
-            host=self.host,
-            port=self.port,
-            bucket=self.bucket,
-            **self.args,
-        )
-
-
-class RemoteInterface(
-    PydanticFieldNoDefaultMixin,
-    pyd.BaseModel,
-    extra="forbid",
-    copy_on_model_validation="none",
-    underscore_attrs_are_private=True,
-):
-    """Remote data lake options."""
-
-    _default_type_description: t.ClassVar[
-        t.Optional[str]
-    ] = "Remote data lakes addresses."
-    _compact_form: t.ClassVar[t.Optional[str]] = "<url>"
-    _default_port_type: t.ClassVar[PiperPortType] = PiperPortType.INPUT
-
-    url: t.Union[str, UrlDataModel] = pyd.Field(
-        ...,
-        description=(
-            "The remote data lake URL. You can user the format "
-            "`s3://user:password@host:port/bucket?kw1=arg1:kw2=arg2`."
-        ),
-    )
-
-    _parsed_url: ParseResult
-
-    @classmethod
-    def __get_validators__(cls):
-        yield cls.validate
-
-    @classmethod
-    def validate(cls, value):
-        if isinstance(value, RemoteInterface):
-            return value
-        if isinstance(value, (str, bytes)):
-            return RemoteInterface(url=str(value))
-        if isinstance(value, t.Mapping):
-            return RemoteInterface(**value)
-        raise ValueError("Invalid remote definition.")
-
-    def __init__(self, **data):
-        from urllib.parse import urlparse
-
-        super().__init__(**data)
-        self._parsed_url = (
-            self.url.get_url()
-            if isinstance(self.url, UrlDataModel)
-            else urlparse(self.url)
-        )
-
-    def get_url(self):
-        return self._parsed_url
-
-
 class ToyDatasetInterface(
     pyd.BaseModel, extra="forbid", copy_on_model_validation="none"
 ):
@@ -709,7 +747,7 @@ class Interval(PydanticFieldWithDefaultMixin, pyd.BaseModel, extra="forbid"):
     """
 
     _default_type_description: t.ClassVar[t.Optional[str]] = "An interval of indexes."
-    _compact_form: t.ClassVar[t.Optional[str]] = "<start>[:<stop>]"
+    _compact_form: t.ClassVar[t.Optional[str]] = "[<start>][:<stop>]"
 
     start: t.Optional[int] = pyd.Field(
         None,
@@ -762,7 +800,7 @@ class ExtendedInterval(Interval):
     ``step`` indices or a string ``start:stop:step``.
     """
 
-    _compact_form: t.ClassVar[t.Optional[str]] = "<start>[:<stop>[:<step>]]"
+    _compact_form: t.ClassVar[t.Optional[str]] = "[<start>][:<stop>][:<step>]"
 
     step: t.Optional[int] = pyd.Field(
         None, description="The slice step, defaults to 1 (can be negative)."
@@ -798,3 +836,128 @@ class ExtendedInterval(Interval):
             raise ValueError(f"Invalid interval: {value}")
 
         return cls(**data)
+
+
+ValueType = t.TypeVar("ValueType")
+
+
+class OutputValueInterface(
+    PydanticFieldNoDefaultMixin,
+    GenericModel,
+    t.Generic[ValueType],
+    extra="forbid",
+    copy_on_model_validation="none",
+    underscore_attrs_are_private=True,
+):
+    """Interface that allows to store a value in a file.
+
+    Examples:
+        How to use it in your command::
+
+        class EasyCommand(PipelimeCommand, title="easy"):
+            output_value: OutputValueInterface[int] = OutputValueInterface.pyd_field()
+
+            def run(self):
+                # this will write the value 42 to the given file
+                self.output_value.set(42)
+    """
+
+    _default_type_description: t.ClassVar[t.Optional[str]] = "An output value."
+    _compact_form: t.ClassVar[t.Optional[str]] = "<file>[,<exists_ok>]"
+    _default_port_type: t.ClassVar[PiperPortType] = PiperPortType.OUTPUT
+    _data: ValueType
+
+    file: Path = pyd.Field(
+        description=(
+            "The output file that will store the value. If the file exists, it will"
+            "be overwritten if `exists_ok` is set to `True`."
+        ),
+    )
+    exists_ok: bool = pyd.Field(
+        False,
+        description=(
+            "If `True`, the output file will be overwritten if it exists. "
+            "If `False`, an error will be raised."
+        ),
+    )
+
+    @pyd.validator("file")
+    def resolve_file(cls, v: Path):
+        # see https://bugs.python.org/issue38671
+        return v.resolve().absolute()
+
+    @classmethod
+    def validate(cls, value):
+        if isinstance(value, OutputValueInterface):
+            return value
+
+        if isinstance(value, (str, bytes, Path)):
+            data = {}
+            raw_data = str(value).split(",")
+            data["file"] = raw_data[0]
+            if len(raw_data) > 1:
+                # NB: the value is left as-is when it is not True nor False
+                # to raise an error on validation
+                data["exists_ok"] = (
+                    True
+                    if raw_data[1].lower() == "true"
+                    else (False if raw_data[1].lower() == "false" else raw_data[1])
+                )
+            value = data
+
+        if isinstance(value, t.Mapping):
+            return OutputValueInterface(**value)
+
+        raise ValueError("Invalid OutputValueInterface definition.")
+
+    @classmethod
+    def __get_validators__(cls):
+        yield cls.validate
+
+    @pyd.validator("exists_ok", always=True)
+    def _check_file_exists(cls, v: bool, values: t.Mapping[str, t.Any]) -> bool:
+        if "file" not in values:
+            # the "file" validator already failed
+            return v
+
+        if v is True:
+            if values["file"].is_dir():
+                raise ValueError(
+                    f"Cannot overwrite the directory: `{values['file']}`. "
+                    "Please provide a file path."
+                )
+
+        if v is False:
+            if values["file"].is_dir():
+                raise ValueError(
+                    f"Trying to overwrite the directory: `{values['file']}`. "
+                    "Please provide a file path and use `exists_ok=True` to overwrite."
+                )
+
+            if values["file"].exists():
+                raise ValueError(
+                    f"Trying to overwrite an existing file: `{values['file']}`. "
+                    "Please use `exists_ok=True` to overwrite."
+                )
+
+        return v
+
+    def get(self) -> ValueType:
+        """Return the stored value."""
+        return self._data
+
+    def set(self, value: ValueType) -> None:
+        """Writes the value to the output file.
+
+        Args:
+            value: The value to write.
+        """
+        self._data = value
+        with open(self.file, "w") as f:
+            json.dump(self._data, f)
+
+    def __repr__(self) -> str:
+        return self.__piper_repr__()
+
+    def __piper_repr__(self) -> str:
+        return self.file.as_posix()
