@@ -1,4 +1,5 @@
 from typing import Optional, Union
+from pathlib import Path
 
 from pipelime.piper.executors.base import NodesGraphExecutor, WatcherNodesGraphExecutor
 from pipelime.piper.executors.naive import NaiveNodesGraphExecutor
@@ -17,7 +18,7 @@ class NodesGraphExecutorFactory:
         cls,
         type_: Optional[str] = None,
         node_prefix: str = "",
-        watch: Union[bool, str] = False,
+        watch: Union[bool, str, Path] = False,
         task: Optional[TrackedTask] = None,
         **kwargs,
     ) -> NodesGraphExecutor:
@@ -26,9 +27,9 @@ class NodesGraphExecutorFactory:
         )
         executor.task = task
 
-        if watch:
+        if watch:  # pragma: no cover
             executor = WatcherNodesGraphExecutor(
-                executor, watch if isinstance(watch, str) else None
+                executor, watch if isinstance(watch, (str, Path)) else None
             )
 
         return executor
